xquery version "3.1";

module namespace facets="http://mollenhauer-edition.de/ns/facets";

declare namespace tei="http://www.tei-c.org/ns/1.0";
declare namespace html="http://www.w3.org/1999/xhtml";
declare namespace index="https://mollenhauer-edition.de/index";

import module namespace config="https://sade.textgrid.de/ns/config" at "../../config.xqm";
import module namespace kwic="http://exist-db.org/xquery/kwic";

import module namespace search="http://mollenhauer-edition.de/ns/search" at "search.xqm";
import module namespace lang="https://sade.textgrid.de/ns/lang" at "../../lang.xqm";

declare function facets:get-facets($search)
as element(ol) {
    element ol {
        attribute class {"search-param-group"},
        attribute role {"list"},
        facets:wits($search),
        facets:keywords($search),
        facets:genres($search)
    }
};


declare function facets:wits($search)
as element(li)
{
    let $facet := ft:facets($search, "wit", ())
    return
        element li {
            attribute class { "search-param" },
            element h2 { lang:translate("witnesses") },
            element ol {
                attribute class {"search-list"},
                attribute role {"list"},
                map:for-each($facet, function($view, $count) {
                    facets:facet("wit", $view, $count)
                })
            }
        }
};

declare function facets:keywords($search)
as element(li)
{
    let $facet := ft:facets($search, "keyword", ())
    return
        element li {
            attribute class { "search-param" },
            element h2 { lang:translate("keywords") },
            element ol {
                attribute class {"search-list"},
                attribute role {"list"},
                map:for-each($facet, function($keyword, $count) {
                    facets:facet("keyword", $keyword, $count)
                })
            }
        }
};

declare function facets:genres($search)
as element(li)
{
    let $facet := ft:facets($search, "genre", ())
    return
        element li {
            attribute class { "search-param" },
            element h2 { lang:translate("genre") },
            element ol {
                attribute class {"search-list"},
                attribute role {"list"},
                map:for-each($facet, function($genre, $count) {
                    facets:facet("genre", $genre, $count)
                })
            }
        }
};

declare function facets:facet($facet as xs:string, $value as xs:string, $count as xs:integer?)
as element(li)
{
    let $active-values := request:get-parameter($facet, "") => tokenize()
    let $display-count := string-join(("(", $count, " ",
        if ($count = 1) then
            lang:translate("textSg")
        else
            lang:translate("textPl"),
        ")"), "")
    return
        if ($active-values => filter(function($f as xs:string) {$f eq $value})) then
            element li {
                element a {
                    attribute class {"search-facet search-facet-active"},
                    attribute href {
                        "search.html?" ||
                        facets:remove-from-query($facet, $value)
                    },
                    element span {
                        attribute class {"fa fa-minus-square"}
                    },
                    element span {
                        $value, $display-count
                    }
                }
            }
        else
            element li {
                element a {
                    attribute class {"search-facet"},
                    attribute href {
                        "search.html?" ||
                        facets:add-to-query($facet, $value)
                    },
                    element span {
                        attribute class {"fa fa-plus-square"}
                    },
                    element span {
                        $value, $display-count
                    }
                }
            }
};

declare function facets:add-to-query($facet as xs:string, $value as xs:string)
as xs:string
{
    let $facet-param := $facet || "="
    let $query := request:get-query-string()
    let $params := $query => tokenize("&amp;")
    let $values := $params => filter(function($param as xs:string) {contains($param, $facet-param)})
    return
        if (empty($values)) then
            $query || "&amp;" || $facet-param || $value
        else
            string-join(
                for $param in $params
                where not(contains($param, $facet-param)) return
                    $param,
                "&amp;"
            )
            || "&amp;" || $values || "+" || $value
};

declare function facets:remove-from-query($facet as xs:string, $value as xs:string) {
    let $facet-param := $facet || "="
    let $query := request:get-query-string()
    let $params := $query => tokenize("&amp;")
    let $values := $params => filter(function($param as xs:string) {contains($param, $facet-param)})
    let $all-values := $values => substring-after($facet-param) => tokenize("\+")
    let $cleared-values := $all-values => filter(function($f as xs:string?) {not($f eq $value)})
    let $query-values := string-join($cleared-values, "+")
    return
        string-join(
            for $param in $params
            where not(contains($param, $facet-param)) return
                $param,
            "&amp;"
        )
        || (if (empty($cleared-values)) then "" else "&amp;" || $facet-param || $query-values)
};